"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isElementWebComponent = exports.defineAllRequiredWebComponents = void 0;
const KSLAddButtonElement_1 = require("./KSLAddButtonElement");
const KSLContainerElement_1 = require("./KSLContainerElement");
const KSLButtonElement_1 = require("./KSLButtonElement");
const KSLIconElement_1 = require("./KSLIconElement");
const KSLTooltipElement_1 = require("./KSLTooltipElement");
const KSLHighlightElement_1 = require("./KSLHighlightElement");
const KSLPopoverElement_1 = require("./KSLPopoverElement");
const webComponents = [
    KSLIconElement_1.KSLIconElement,
    KSLButtonElement_1.KSLButtonElement,
    KSLAddButtonElement_1.KSLAddButtonElement,
    KSLHighlightElement_1.KSLHighlightElement,
    KSLContainerElement_1.KSLContainerElement,
    KSLTooltipElement_1.KSLTooltipElement,
    KSLPopoverElement_1.KSLPopoverElement,
];
const webComponentTags = webComponents.map((component) => component.is);
/**
 * Define all web components if it is possible.
 *
 * @returns {Promise<void[]>}
 */
function defineAllRequiredWebComponents() {
    const allDefined = webComponents.map((webComponent) => webComponent.define());
    return Promise.all(allDefined);
}
exports.defineAllRequiredWebComponents = defineAllRequiredWebComponents;
/**
 * Check if element is a web component based on its tag name.
 *
 * @param {HTMLElement} element
 * @returns {boolean}
 */
function isElementWebComponent(element) {
    return webComponentTags.includes(element.tagName.toLowerCase());
}
exports.isElementWebComponent = isElementWebComponent;
//# sourceMappingURL=components.js.map