export declare enum HighlightType {
    None = "",
    Element = "element",
    ContentComponent = "content-component",
    ContentItem = "content-item"
}
/**
 * Find all descendant HTML elements that could be augmented (have highlights or add buttons near them).
 *
 * @param {HTMLElement | Document} node
 * @returns {NodeListOf<HTMLElement>}
 */
export declare function getAugmentableDescendants(node: HTMLElement | Document): NodeListOf<HTMLElement>;
/**
 * Checks if HTML element could be augmented (have highlights or add buttons near them).
 *
 * @param {HTMLElement | null} element
 * @returns {boolean}
 */
export declare function isElementAugmentable(element: HTMLElement | null): boolean;
/**
 * Check if node should have highlights based on its data-attributes.
 *
 * @param {HTMLElement | null} element
 * @returns {boolean}
 */
export declare function shouldElementHaveHighlight(element: HTMLElement | null): boolean;
/**
 * Check if node should have a add button based on its data-attributes.
 *
 * @param {HTMLElement | null} element
 * @returns {boolean}
 */
export declare function shouldElementHaveAddButton(element: HTMLElement | null): boolean;
/**
 * Get HighlightType based on the HTML element data-attributes.
 *
 * @param {HTMLElement | null} element
 * @returns {HighlightType}
 */
export declare function getHighlightTypeForElement(element: HTMLElement | null): HighlightType;
